/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kunifiedpush.distributor.Management.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "managementadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class ManagementAdaptor
 */

ManagementAdaptor::ManagementAdaptor(KUnifiedPush::Distributor *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

ManagementAdaptor::~ManagementAdaptor()
{
    // destructor
}

QString ManagementAdaptor::errorMessage()
{
    // handle method call org.kde.kunifiedpush.Management.errorMessage
    return parent()->errorMessage();
}

void ManagementAdaptor::forceUnregisterClient(const QString &clientToken)
{
    // handle method call org.kde.kunifiedpush.Management.forceUnregisterClient
    parent()->forceUnregisterClient(clientToken);
}

QVariantMap ManagementAdaptor::pushProviderConfiguration(const QString &pushProviderId)
{
    // handle method call org.kde.kunifiedpush.Management.pushProviderConfiguration
    return parent()->pushProviderConfiguration(pushProviderId);
}

QString ManagementAdaptor::pushProviderId()
{
    // handle method call org.kde.kunifiedpush.Management.pushProviderId
    return parent()->pushProviderId();
}

QList<KUnifiedPush::ClientInfo> ManagementAdaptor::registeredClients()
{
    // handle method call org.kde.kunifiedpush.Management.registeredClients
    return parent()->registeredClients();
}

void ManagementAdaptor::setPushProvider(const QString &pushProvider, const QVariantMap &config)
{
    // handle method call org.kde.kunifiedpush.Management.setPushProvider
    parent()->setPushProvider(pushProvider, config);
}

int ManagementAdaptor::status()
{
    // handle method call org.kde.kunifiedpush.Management.status
    return parent()->status();
}


#include "moc_managementadaptor.cpp"
