/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.unifiedpush.Distributor1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "distributor1adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class Distributor1Adaptor
 */

Distributor1Adaptor::Distributor1Adaptor(KUnifiedPush::Distributor *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

Distributor1Adaptor::~Distributor1Adaptor()
{
    // destructor
}

QString Distributor1Adaptor::Register(const QString &serviceName, const QString &token, const QString &description, QString &registrationResultReason)
{
    // handle method call org.unifiedpush.Distributor1.Register
    return parent()->Register(serviceName, token, description, registrationResultReason);
}

void Distributor1Adaptor::Unregister(const QString &token)
{
    // handle method call org.unifiedpush.Distributor1.Unregister
    parent()->Unregister(token);
}


#include "moc_distributor1adaptor.cpp"
