/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.unifiedpush.Connector1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef CONNECTOR1ADAPTOR_H
#define CONNECTOR1ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "connector_p.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.unifiedpush.Connector1
 */
class Connector1Adaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.unifiedpush.Connector1")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.unifiedpush.Connector1\">\n"
"    <method name=\"Message\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"token\"/>\n"
"      <arg direction=\"in\" type=\"ay\" name=\"message\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"messageIdentifier\"/>\n"
"      <annotation value=\"true\" name=\"org.freedesktop.DBus.Method.NoReply\"/>\n"
"    </method>\n"
"    <method name=\"NewEndpoint\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"token\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"endpoint\"/>\n"
"      <annotation value=\"true\" name=\"org.freedesktop.DBus.Method.NoReply\"/>\n"
"    </method>\n"
"    <method name=\"Unregistered\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"token\"/>\n"
"      <annotation value=\"true\" name=\"org.freedesktop.DBus.Method.NoReply\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    Connector1Adaptor(KUnifiedPush::ConnectorPrivate *parent);
    ~Connector1Adaptor() override;

    inline KUnifiedPush::ConnectorPrivate *parent() const
    { return static_cast<KUnifiedPush::ConnectorPrivate *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    Q_NOREPLY void Message(const QString &token, const QByteArray &message, const QString &messageIdentifier);
    Q_NOREPLY void NewEndpoint(const QString &token, const QString &endpoint);
    Q_NOREPLY void Unregistered(const QString &token);
Q_SIGNALS: // SIGNALS
};

#endif
