/****************************************************************************
** Meta object code from reading C++ file 'managementadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "managementadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'managementadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17ManagementAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto ManagementAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN17ManagementAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ManagementAdaptor",
        "D-Bus Interface",
        "org.kde.kunifiedpush.Management",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.kunifiedpush.Management\">\n    <metho"
        "d name=\"status\">\n      <arg direction=\"out\" type=\"i\" name=\""
        "status\"/>\n    </method>\n    <signal name=\"statusChanged\"/>\n "
        "   <method name=\"errorMessage\">\n      <arg direction=\"out\" ty"
        "pe=\"s\" name=\"errorMessage\"/>\n    </method>\n    <signal name="
        "\"errorMessageChanged\"/>\n    <method name=\"pushProviderId\">\n "
        "     <arg direction=\"out\" type=\"s\" name=\"status\"/>\n    </me"
        "thod>\n    <method name=\"pushProviderConfiguration\">\n      <arg"
        " direction=\"in\" type=\"s\" name=\"pushProviderId\"/>\n      <arg"
        " direction=\"out\" type=\"a{sv}\" name=\"config\"/>\n      <annota"
        "tion value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName."
        "Out0\"/>\n    </method>\n    <method name=\"setPushProvider\">\n  "
        "    <arg direction=\"in\" type=\"s\" name=\"pushProvider\"/>\n    "
        "  <arg direction=\"in\" type=\"a{sv}\" name=\"config\"/>\n      <a"
        "nnotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtType"
        "Name.In1\"/>\n      <annotation value=\"true\" name=\"org.freedesk"
        "top.DBus.Method.NoReply\"/>\n    </method>\n    <signal name=\"pus"
        "hProviderChanged\"/>\n    <method name=\"registeredClients\">\n   "
        "   <arg direction=\"out\" type=\"a(sss)\" name=\"clients\"/>\n    "
        "  <annotation value=\"QList&lt;KUnifiedPush::ClientInfo&gt;\" name"
        "=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n    </method>\n    <m"
        "ethod name=\"forceUnregisterClient\">\n      <arg direction=\"in\""
        " type=\"s\" name=\"clientToken\"/>\n      <annotation value=\"true"
        "\" name=\"org.freedesktop.DBus.Method.NoReply\"/>\n    </method>\n"
        "    <signal name=\"registeredClientsChanged\"/>\n  </interface>\n",
        "errorMessageChanged",
        "",
        "pushProviderChanged",
        "registeredClientsChanged",
        "statusChanged",
        "errorMessage",
        "forceUnregisterClient",
        "Q_NOREPLY",
        "clientToken",
        "pushProviderConfiguration",
        "QVariantMap",
        "pushProviderId",
        "registeredClients",
        "QList<KUnifiedPush::ClientInfo>",
        "setPushProvider",
        "pushProvider",
        "config",
        "status"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'errorMessageChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pushProviderChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'registeredClientsChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'errorMessage'
        QtMocHelpers::SlotData<QString()>(10, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'forceUnregisterClient'
        QtMocHelpers::SlotData<void(const QString &)>(11, 12, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'pushProviderConfiguration'
        QtMocHelpers::SlotData<QVariantMap(const QString &)>(14, 6, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'pushProviderId'
        QtMocHelpers::SlotData<QString()>(16, 6, QMC::AccessPublic, QMetaType::QString),
        // Slot 'registeredClients'
        QtMocHelpers::SlotData<QList<KUnifiedPush::ClientInfo>()>(17, 6, QMC::AccessPublic, 0x80000000 | 18),
        // Slot 'setPushProvider'
        QtMocHelpers::SlotData<void(const QString &, const QVariantMap &)>(19, 12, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { 0x80000000 | 15, 21 },
        }}),
        // Slot 'status'
        QtMocHelpers::SlotData<int()>(22, 6, QMC::AccessPublic, QMetaType::Int),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<ManagementAdaptor, qt_meta_tag_ZN17ManagementAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ManagementAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17ManagementAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17ManagementAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17ManagementAdaptorE_t>.metaTypes,
    nullptr
} };

void ManagementAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ManagementAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->errorMessageChanged(); break;
        case 1: _t->pushProviderChanged(); break;
        case 2: _t->registeredClientsChanged(); break;
        case 3: _t->statusChanged(); break;
        case 4: { QString _r = _t->errorMessage();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->forceUnregisterClient((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: { QVariantMap _r = _t->pushProviderConfiguration((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->pushProviderId();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 8: { QList<KUnifiedPush::ClientInfo> _r = _t->registeredClients();
            if (_a[0]) *reinterpret_cast<QList<KUnifiedPush::ClientInfo>*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setPushProvider((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 10: { int _r = _t->status();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ManagementAdaptor::*)()>(_a, &ManagementAdaptor::errorMessageChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ManagementAdaptor::*)()>(_a, &ManagementAdaptor::pushProviderChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ManagementAdaptor::*)()>(_a, &ManagementAdaptor::registeredClientsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ManagementAdaptor::*)()>(_a, &ManagementAdaptor::statusChanged, 3))
            return;
    }
}

const QMetaObject *ManagementAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ManagementAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17ManagementAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int ManagementAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void ManagementAdaptor::errorMessageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ManagementAdaptor::pushProviderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ManagementAdaptor::registeredClientsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ManagementAdaptor::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
