/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.unifiedpush.Connector2.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef CONNECTOR2ADAPTOR_H
#define CONNECTOR2ADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "connector_p.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.unifiedpush.Connector2
 */
class Connector2Adaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.unifiedpush.Connector2")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.unifiedpush.Connector2\">\n"
"    <method name=\"Message\">\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"args\"/>\n"
"      <arg direction=\"out\" type=\"a{sv}\" name=\"res\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"NewEndpoint\">\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"args\"/>\n"
"      <arg direction=\"out\" type=\"a{sv}\" name=\"res\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"    <method name=\"Unregistered\">\n"
"      <arg direction=\"in\" type=\"a{sv}\" name=\"args\"/>\n"
"      <arg direction=\"out\" type=\"a{sv}\" name=\"res\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n"
"      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    Connector2Adaptor(KUnifiedPush::ConnectorPrivate *parent);
    ~Connector2Adaptor() override;

    inline KUnifiedPush::ConnectorPrivate *parent() const
    { return static_cast<KUnifiedPush::ConnectorPrivate *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    QVariantMap Message(const QVariantMap &args);
    QVariantMap NewEndpoint(const QVariantMap &args);
    QVariantMap Unregistered(const QVariantMap &args);
Q_SIGNALS: // SIGNALS
};

#endif
