/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.unifiedpush.Distributor2.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "distributor2adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class Distributor2Adaptor
 */

Distributor2Adaptor::Distributor2Adaptor(KUnifiedPush::Distributor *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

Distributor2Adaptor::~Distributor2Adaptor()
{
    // destructor
}

QVariantMap Distributor2Adaptor::Register(const QVariantMap &args)
{
    // handle method call org.unifiedpush.Distributor2.Register
    return parent()->Register(args);
}

QVariantMap Distributor2Adaptor::Unregister(const QVariantMap &args)
{
    // handle method call org.unifiedpush.Distributor2.Unregister
    return parent()->Unregister(args);
}


#include "moc_distributor2adaptor.cpp"
