/****************************************************************************
** Meta object code from reading C++ file 'managementinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "managementinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'managementinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN37OrgKdeKunifiedpushManagementInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeKunifiedpushManagementInterface::qt_create_metaobjectdata<qt_meta_tag_ZN37OrgKdeKunifiedpushManagementInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeKunifiedpushManagementInterface",
        "errorMessageChanged",
        "",
        "pushProviderChanged",
        "registeredClientsChanged",
        "statusChanged",
        "errorMessage",
        "QDBusPendingReply<QString>",
        "forceUnregisterClient",
        "Q_NOREPLY",
        "clientToken",
        "pushProviderConfiguration",
        "QDBusPendingReply<QVariantMap>",
        "pushProviderId",
        "registeredClients",
        "QDBusPendingReply<QList<KUnifiedPush::ClientInfo>>",
        "setPushProvider",
        "pushProvider",
        "QVariantMap",
        "config",
        "status",
        "QDBusPendingReply<int>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'errorMessageChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pushProviderChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'registeredClientsChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'errorMessage'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(6, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'forceUnregisterClient'
        QtMocHelpers::SlotData<void(const QString &)>(8, 9, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'pushProviderConfiguration'
        QtMocHelpers::SlotData<QDBusPendingReply<QVariantMap>(const QString &)>(11, 2, QMC::AccessPublic, 0x80000000 | 12, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'pushProviderId'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(13, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'registeredClients'
        QtMocHelpers::SlotData<QDBusPendingReply<QList<KUnifiedPush::ClientInfo> >()>(14, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'setPushProvider'
        QtMocHelpers::SlotData<void(const QString &, const QVariantMap &)>(16, 9, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'status'
        QtMocHelpers::SlotData<QDBusPendingReply<int>()>(20, 2, QMC::AccessPublic, 0x80000000 | 21),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeKunifiedpushManagementInterface, qt_meta_tag_ZN37OrgKdeKunifiedpushManagementInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeKunifiedpushManagementInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37OrgKdeKunifiedpushManagementInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37OrgKdeKunifiedpushManagementInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN37OrgKdeKunifiedpushManagementInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeKunifiedpushManagementInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeKunifiedpushManagementInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->errorMessageChanged(); break;
        case 1: _t->pushProviderChanged(); break;
        case 2: _t->registeredClientsChanged(); break;
        case 3: _t->statusChanged(); break;
        case 4: { QDBusPendingReply<QString> _r = _t->errorMessage();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->forceUnregisterClient((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: { QDBusPendingReply<QVariantMap> _r = _t->pushProviderConfiguration((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<QString> _r = _t->pushProviderId();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<QList<KUnifiedPush::ClientInfo>> _r = _t->registeredClients();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QList<KUnifiedPush::ClientInfo>>*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setPushProvider((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2]))); break;
        case 10: { QDBusPendingReply<int> _r = _t->status();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKunifiedpushManagementInterface::*)()>(_a, &OrgKdeKunifiedpushManagementInterface::errorMessageChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKunifiedpushManagementInterface::*)()>(_a, &OrgKdeKunifiedpushManagementInterface::pushProviderChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKunifiedpushManagementInterface::*)()>(_a, &OrgKdeKunifiedpushManagementInterface::registeredClientsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKunifiedpushManagementInterface::*)()>(_a, &OrgKdeKunifiedpushManagementInterface::statusChanged, 3))
            return;
    }
}

const QMetaObject *OrgKdeKunifiedpushManagementInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeKunifiedpushManagementInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN37OrgKdeKunifiedpushManagementInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeKunifiedpushManagementInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void OrgKdeKunifiedpushManagementInterface::errorMessageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OrgKdeKunifiedpushManagementInterface::pushProviderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OrgKdeKunifiedpushManagementInterface::registeredClientsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OrgKdeKunifiedpushManagementInterface::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
