/****************************************************************************
** Meta object code from reading C++ file 'distributor2adaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "distributor2adaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'distributor2adaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19Distributor2AdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto Distributor2Adaptor::qt_create_metaobjectdata<qt_meta_tag_ZN19Distributor2AdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Distributor2Adaptor",
        "D-Bus Interface",
        "org.unifiedpush.Distributor2",
        "D-Bus Introspection",
        "  <interface name=\"org.unifiedpush.Distributor2\">\n    <method n"
        "ame=\"Register\">\n      <arg direction=\"in\" type=\"a{sv}\" name"
        "=\"args\"/>\n      <arg direction=\"out\" type=\"a{sv}\" name=\"re"
        "s\"/>\n      <annotation value=\"QVariantMap\" name=\"org.qtprojec"
        "t.QtDBus.QtTypeName.In0\"/>\n      <annotation value=\"QVariantMap"
        "\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n    </method>\n"
        "    <method name=\"Unregister\">\n      <arg direction=\"in\" type"
        "=\"a{sv}\" name=\"args\"/>\n      <arg direction=\"out\" type=\"a{"
        "sv}\" name=\"res\"/>\n      <annotation value=\"QVariantMap\" name"
        "=\"org.qtproject.QtDBus.QtTypeName.In0\"/>\n      <annotation valu"
        "e=\"QVariantMap\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\"/>\n"
        "    </method>\n  </interface>\n",
        "Register",
        "QVariantMap",
        "",
        "args",
        "Unregister"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'Register'
        QtMocHelpers::SlotData<QVariantMap(const QVariantMap &)>(5, 7, QMC::AccessPublic, 0x80000000 | 6, {{
            { 0x80000000 | 6, 8 },
        }}),
        // Slot 'Unregister'
        QtMocHelpers::SlotData<QVariantMap(const QVariantMap &)>(9, 7, QMC::AccessPublic, 0x80000000 | 6, {{
            { 0x80000000 | 6, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<Distributor2Adaptor, qt_meta_tag_ZN19Distributor2AdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Distributor2Adaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19Distributor2AdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19Distributor2AdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19Distributor2AdaptorE_t>.metaTypes,
    nullptr
} };

void Distributor2Adaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Distributor2Adaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QVariantMap _r = _t->Register((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 1: { QVariantMap _r = _t->Unregister((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *Distributor2Adaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Distributor2Adaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19Distributor2AdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int Distributor2Adaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
