/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.unifiedpush.Connector2.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef CONNECTOR2IFACE_H
#define CONNECTOR2IFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.unifiedpush.Connector2
 */
class OrgUnifiedpushConnector2Interface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.unifiedpush.Connector2"; }

public:
    OrgUnifiedpushConnector2Interface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgUnifiedpushConnector2Interface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QVariantMap> Message(const QVariantMap &args)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(args);
        return asyncCallWithArgumentList(QStringLiteral("Message"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> NewEndpoint(const QVariantMap &args)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(args);
        return asyncCallWithArgumentList(QStringLiteral("NewEndpoint"), argumentList);
    }

    inline QDBusPendingReply<QVariantMap> Unregistered(const QVariantMap &args)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(args);
        return asyncCallWithArgumentList(QStringLiteral("Unregistered"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace unifiedpush {
    using Connector2 = ::OrgUnifiedpushConnector2Interface;
  }
}

#endif
