/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.unifiedpush.Connector1.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "connector1adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class Connector1Adaptor
 */

Connector1Adaptor::Connector1Adaptor(KUnifiedPush::ConnectorPrivate *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

Connector1Adaptor::~Connector1Adaptor()
{
    // destructor
}

void Connector1Adaptor::Message(const QString &token, const QByteArray &message, const QString &messageIdentifier)
{
    // handle method call org.unifiedpush.Connector1.Message
    parent()->Message(token, message, messageIdentifier);
}

void Connector1Adaptor::NewEndpoint(const QString &token, const QString &endpoint)
{
    // handle method call org.unifiedpush.Connector1.NewEndpoint
    parent()->NewEndpoint(token, endpoint);
}

void Connector1Adaptor::Unregistered(const QString &token)
{
    // handle method call org.unifiedpush.Connector1.Unregistered
    parent()->Unregistered(token);
}


#include "moc_connector1adaptor.cpp"
